@@ -10,13 +10,13 @@ import dice.utilities
 
 __all__ = ['roll', 'ParseException']
 __author__ = "Sam Clements <sam@borntyping.co.uk>"
-__version__ = '0.3.2'
+__version__ = '0.4.2'
 
 
 def roll(string, single=True, verbose=False):
     """Parses and evaluates a dice expression"""
     ast = dice.grammar.expression.parseString(string, parseAll=True)
-    result = [element.evaluate(verbose=verbose) for element in ast]
+    result = [element.evaluate_cached(verbose=verbose) for element in ast]
     if single:
         return dice.utilities.single(result)
     return result
